import re
from ._video import Video
from ._videobulk import _VideoBulk
from ._auxiliary import _filter, _src


class Extras:

    def __init__(self):
        pass

    @property
    def trending(self):
        """
        :return: < video object > of #1 on trending video
        """
        raw = _src(f'https://www.youtube.com/feed/trending')
        data = re.findall(r"videoId\":\"(.*?)\"", raw)
        return Video(data[0]) if data else None

    @property
    def music(self):
        """
        :return: list of < video object > of trending music videos
        """
        raw = _src(f'https://www.youtube.com/feed/music')
        data = re.findall(r"videoId\":\"(.*?)\"", raw)
        return _VideoBulk(_filter(data)) if data else None

    @property
    def gaming(self):
        """
        :return: list of < video object > of trending gaming videos
        """
        raw = _src(f'https://www.youtube.com/gaming')
        data = re.findall(r"videoId\":\"(.*?)\"", raw)
        return _VideoBulk(_filter(data)) if data else None

    @property
    def news(self):
        """
        :return: list of < video object > of trending news videos
        """
        raw = _src(f'https://www.youtube.com/news')
        data = re.findall(r"videoId\":\"(.*?)\"", raw)
        return _VideoBulk(_filter(data)) if data else None

    @property
    def livestream(self):
        """
        :return: list of < video object > of trending livestreams
        """
        raw = _src(f'https://www.youtube.com/live')
        data = re.findall(r"videoId\":\"(.*?)\"", raw)
        return _VideoBulk(_filter(data)) if data else None

    @property
    def learning(self):
        """
        :return: list of < video object > of trending educational videos
        """
        raw = _src(f'https://www.youtube.com/learning')
        data = re.findall(r"videoId\":\"(.*?)\"", raw)
        return _VideoBulk(_filter(data)) if data else None

    @property
    def sports(self):
        """
        :return: list of < video object > of trending sports videos
        """
        raw = _src(f'https://www.youtube.com/sports')
        data = re.findall(r"videoId\":\"(.*?)\"", raw)
        return _VideoBulk(_filter(data)) if data else None



import concurrent.futures


class _Thread:

    def __init__(self):
        pass

    @staticmethod
    def run(function, args: list) -> list:
        num = len(args) if len(args) > 0 else 1
        if args:
            with concurrent.futures.ThreadPoolExecutor(max_workers=num) as exec:
                return list(exec.map(function, args))

        



import re
from ._video import Video
from ._auxiliary import _src, _filter


class Playlist:
    def __init__(self, playlist_id: str):
        """
        :param str playlist_id: the _id of the playlist
        """
        if 'youtube.com' in playlist_id:
            self.id = re.findall(r'=(.*)', playlist_id)[0]
        else:
            self.id = playlist_id

    @property
    def name(self):
        """
        :return: the name of the playlist
        """
        raw = _src(f'https://www.youtube.com/playlist?list={self.id}')
        name_data = re.findall(r"{\"title\":\"(.*?)\"", raw)
        return name_data[0] if name_data else None

    @property
    def url(self):
        """
        :return: url of the playlist
        """
        return f'https://www.youtube.com/playlist?list={self.id}'

    @property
    def video_count(self):
        """
        :return: total number of videos in that playlist
        """
        raw = _src(f'https://www.youtube.com/playlist?list={self.id}')
        video_count = re.findall(r"stats\":\[{\"runs\":\[{\"text\":\"(.*?)\"", raw)
        return video_count[0] if video_count else None

    def videos(self, limit: int = None):
        """
        :param int limit: number of videos the user want from the playlist
        :return: list of < video objects > for each video in the playlist (consider limit)
        """

        raw = _src(f'https://www.youtube.com/playlist?list={self.id}')
        videos = re.findall(r"videoId\":\"(.*?)\"", raw)
        pure = _filter(limit=limit, iterable=videos)
        return [Video(item) for item in pure]

    @property
    def thumbnail(self):
        """
        :return: url of the thumbnail of the playlist
        """
        raw = _src(f'https://www.youtube.com/playlist?list={self.id}')
        thumbnails = re.findall(r"og:image\" content=\"(.*?)\?", raw)
        return thumbnails[0] if thumbnails else None
    
    @property
    def info(self):
        """
        :return: a dict containing playlist info
        dict = {
                'name': -> str,
                'url': -> str,
                'video_count': -> int,
                'videos': -> bulk,
                'thumbnail': -> str,
            }
        """
        raw = _src(f'https://www.youtube.com/playlist?list={self.id}')
        name_data = re.findall(r"{\"title\":\"(.*?)\"", raw)
        video_count_data = re.findall(r"stats\":\[{\"runs\":\[{\"text\":\"(.*?)\"", raw)
        thumbnails = re.findall(r"og:image\" content=\"(.*?)\?", raw)

        return {

            'name': name_data[0] if len(name_data) != 0 else None,
            'video_count': video_count_data[0] if video_count_data else None,
            'videos': _filter(re.findall(r"videoId\":\"(.*?)\"", raw)),
            'thumbnail': thumbnails[0] if len(thumbnails) != 0 else None,
            'url': f'https://www.youtube.com/playlist?list={self.id}'
        }
        


import re
from ._threads import _Thread
from ._auxiliary import _duration, _src


class Video:

    def __init__(self, videoId: str):
        """
        :param videoId: video id or the url of the video
        """
        if 'watch?v=' in videoId:
            self._url = videoId
            self._id = re.findall(r"v=(.*)", videoId)[0]

        elif 'youtu.be/' in videoId:
            idL = re.findall(r"youtu\.be/(.*)", videoId)
            self._url = f'https://www.youtube.com/watch?v={idL[0]}'
            self._id = re.findall(r"/(.*)", videoId)[0]
        else:
            self._url = f'https://www.youtube.com/watch?v={videoId}'
            self._id = videoId

    @property
    def url(self):
        return self._url

    @property
    def id(self):
        return self._id

    @property
    def title(self):
        """
        :return: the title of the video
        """
        raw = _src(self._url)
        data = re.findall(r"\"title\":\"(.*?)\"", raw)
        return data[0] if data else None

    @property
    def views(self):
        """
        :return: total views the video got so far
        """
        raw = _src(self._url)
        data = re.findall(
            r"\"videoViewCountRenderer\":{\"viewCount\":{\"simpleText\":\"(.*?)\"",
            raw
        )
        return data[0][:-6] if data else None

    @property
    def likes(self):
        """
        :return: total likes the video got so far
        """
        raw = _src(self._url)
        data = re.findall(
            r"toggledText\":{\"accessibility\":{\"accessibilityData\":{\"label\":\"(.*?) ",
            raw
        )
        return data[0] if data else None

    @property
    def dislikes(self):
        raise DeprecationWarning("This property is deprecated as YouTube is slowly removing public dislike counts.")

    @property
    def duration(self):
        """
        :return: total duration of  the video in 00h 00m 00s
        """
        raw = _src(self._url)
        data = re.findall(r"approxDurationMs\":\"(.*?)\"", raw)
        return _duration(int(int(data[0]) / 1000)) if data else None

    @property
    def date(self):
        """
        :return: the date on which the video has been uploaded
        """
        raw = _src(self._url)
        data = re.findall(r"uploadDate\":\"(.*?)\"", raw)
        return data[0] if data else None

    @property
    def author(self):
        """
        :return: the id of the channel from which the video belongs
        """
        raw = _src(self._url)
        data = re.findall(r"channelIds\":\[\"(.*?)\"", raw)
        return data[0] if data else None

    @property
    def description(self):
        """
        :return: description provided with the video
        """
        raw = _src(self._url)
        data = re.findall(r"shortDescription\":\"(.*)\",\"isCrawlable", raw)
        return data[0].replace('\\n', '') if data else None

    @property
    def thumbnail(self):
        """
        :return: _url of the thumbnail of the video
        """
        raw = _src(self._url)
        data = re.findall(
            r"playerMicroformatRenderer\":{\"thumbnail\":{\"thumbnails\":\[{\"url\":\"(.*?)\"",
            raw
        )
        return data[0] if data else None

    @property
    def tags(self):
        """
        :return: list of tags used in video meta-data
        """
        raw = _src(self._url)
        data = re.findall(r"<meta name=\"keywords\" content=\"(.*?)\">", raw)
        return data[0].split(',') if data else None

    @property
    def info(self):
        """
        :return: dict containing the the whole info of the video
        dict = {

            'title': -> str,
            'id': -> str,
            'views': -> str,
            'likes': -> str,
            'dislikes': -> str,
            'duration': -> str,
            'author': -> str,
            'uploaded': -> str,
            'url': -> str,
            'thumbnail': -> str,
            'tags': -> list,
        }
        """
        raw = _src(self._url)

        def _get_data(pattern):
            data = re.findall(pattern, raw)
            return data[0] if len(data) > 0 else None

        patterns = [

            r"\"title\":\"(.*?)\"",
            r"\"videoViewCountRenderer\":{\"viewCount\":{\"simpleText\":\"(.*?)\"",
            r"toggledText\":{\"accessibility\":{\"accessibilityData\":{\"label\":\"(.*?) ",
            r"approxDurationMs\":\"(.*?)\"",
            r"author\":\"(.*?)\"",
            r"uploadDate\":\"(.*?)\"",
            r"playerMicroformatRenderer\":{\"thumbnail\":{\"thumbnails\":\[{\"url\":\"(.*?)\"",
            r"<meta name=\"keywords\" content=\"(.*?)\">"

        ]

        ls = _Thread.run(_get_data, patterns)

        return {

            'title': ls[0],
            'id': self._id,
            'views': ls[1][:-6],
            'likes': ls[2],
            'duration': _duration(int(int(ls[3]) / 1000)),
            'author': ls[4],
            'uploaded': ls[5],
            'url': self._url,
            'thumbnail': ls[6],
            'tags': ls[7].split(','),
        }



import re
from ._auxiliary import _src
from ._threads import _Thread


class _PlaylistBulk:

    def __init__(self, iter: list):
        self._ls = iter

    @property
    def urls(self):
        head = 'https://www.youtube.com/playlist?list='
        return [f'{head}{item}' for item in self._ls]

    
    @property
    def _sources(self):
        def get_page(url):
            return _src(url)
        return _Thread.run(get_page, self.urls)

    @property
    def names(self):
        pattern = r"title\":\"(.*?)\""
        return [re.findall(pattern, item)[0] for item in self._sources]

    @property
    def video_counts(self):
        pattern = r"stats\":\[{\"runs\":\[{\"text\":\"(.*?)\""
        temp = [re.findall(pattern, item) for item in self._sources]
        return [item[0] if item else None for item in temp]

    @property
    def thumbnails(self):
        pattern = r"og:image\" content=\"(.*?)\?"
        temp = [re.findall(pattern, item) for item in self._sources]
        return [item[0] if item else None for item in temp]


import re
from ._threads import _Thread
from ._auxiliary import _src, _duration


class _VideoBulk:

    def __init__(self, iterable: list):
        self._ls = iterable

    @property
    def _sources(self):
        head = 'https://www.youtube.com/watch?v='
        urls = [f'{head}{item}/about' for item in self._ls]

        def get_page(url):
            return _src(url)

        return _Thread.run(get_page, urls)

    @property
    def ids(self):
        return self._ls

    @property
    def urls(self):
        head = 'https://www.youtube.com/watch?v='
        return [f'{head}{item}' for item in self._ls]

    @property
    def titles(self):
        pattern = r"title\":\"(.*?)\""
        return [re.findall(pattern, item)[0] for item in self._sources]

    @property
    def views(self):
        pattern = r"\"videoViewCountRenderer\":{\"viewCount\":{\"simpleText\":\"(.*?)\""
        temp = [re.findall(pattern, item) for item in self._sources]
        return [item[0][:-6] if item else None for item in temp]

    @property
    def likes(self):
        pattern = r"toggledText\":{\"accessibility\":{\"accessibilityData\":{\"label\":\"(.*?) "
        temp = [re.findall(pattern, item) for item in self._sources]
        return [item[0] if item else None for item in temp]

    @property
    def dislikes(self):
        raise DeprecationWarning("This property is deprecated as YouTube is slowly removing public dislike counts.")

    @property
    def durations(self):
        pattern = r"approxDurationMs\":\"(.*?)\""
        temp = [re.findall(pattern, item) for item in self._sources]
        return [_duration(int(int(item[0]) / 1000)) if item else None for item in temp]

    @property
    def dates(self):
        pattern = r"uploadDate\":\"(.*?)\""
        temp = [re.findall(pattern, item) for item in self._sources]
        return [item[0] if item else None for item in temp]

    @property
    def parents(self):
        pattern = r"channelIds\":\[\"(.*?)\""
        temp = [re.findall(pattern, item) for item in self._sources]
        return [item[0] if item else None for item in temp]

    @property
    def descriptions(self):
        pattern = r"shortDescription\":\"(.*)\",\"isCrawlable"
        temp = [re.findall(pattern, item) for item in self._sources]
        return [item[0].replace('\\n', ' ') if item else None for item in temp]

    @property
    def thumbnails(self):
        pattern = r"playerMicroformatRenderer\":{\"thumbnail\":{\"thumbnails\":\[{\"url\":\"(.*?)\""
        temp = [re.findall(pattern, item) for item in self._sources]
        return [item[0] if item else None for item in temp]

    @property
    def tags(self):
        pattern = r"<meta name=\"keywords\" content=\"(.*?)\">"
        temp = [re.findall(pattern, item) for item in self._sources]
        return [item[0].split(',') if item else None for item in temp]



import re
from ._video import Video
from ._channel import Channel
from ._playlist import Playlist
from ._videobulk import _VideoBulk
from ._channelbulk import _ChannelBulk
from ._playlistbulk import _PlaylistBulk
from ._auxiliary import _parser, _filter, _src


class Search:

    def __init__(self):
        pass

    @staticmethod
    def video(keywords: str):
        """
        :return: < video object > regarding the query
        """
        raw = _src(f'https://www.youtube.com/results?search_query={_parser(keywords)}&sp=EgIQAQ%253D%253D')
        video_ids = re.findall(r"\"videoId\":\"(.*?)\"", raw)
        return Video(video_ids[0]) if video_ids else None

    @staticmethod
    def channel(keywords: str):
        """
        :return: < channel object > regarding the query
        """
        raw = _src(f'https://www.youtube.com/results?search_query={_parser(keywords)}&sp=EgIQAg%253D%253D')
        channel_ids = re.findall(r"{\"channelId\":\"(.*?)\"", raw)
        return Channel(channel_ids[0]) if channel_ids else None

    @staticmethod
    def videos(keywords: str, limit: int):
        """
        :param str keywords: query to be searched on YouTube
        :param int limit: total number of videos to be searched
        :return: list of < video object > of each video regarding the query (consider limit)
        """
        raw = _src(f'https://www.youtube.com/results?search_query={_parser(keywords)}&sp=EgIQAQ%253D%253D')
        raw_ids = re.findall(r"\"videoId\":\"(.*?)\"", raw)
        pureList = _filter(limit=limit, iterable=raw_ids)
        return _VideoBulk(pureList) if pureList else None

    @staticmethod
    def channels(keywords: str, limit: int):
        """
        :param str keywords: query to be searched on YouTube
        :param int limit: total number of channels to be searched
        :return: list of < channel object > of each video regarding the query (consider limit)
        """
        raw = _src(f'https://www.youtube.com/results?search_query={_parser(keywords)}&sp=EgIQAg%253D%253D')
        raw_ids = re.findall(r"{\"channelId\":\"(.*?)\"", raw)
        pureList = _filter(limit=limit, iterable=raw_ids)
        return _ChannelBulk(pureList) if pureList else None

    @staticmethod
    def playlist(keywords: str):
        """
        :return: < playlist object > regarding the query
        """
        raw = _src(f'https://www.youtube.com/results?search_query={_parser(keywords)}&sp=EgIQAw%253D%253D')
        found = re.findall(r"playlistId\":\"(.*?)\"", raw)
        return Playlist(found[0]) if found else None

    @staticmethod
    def playlists(keywords: str, limit: int):
        """
        :param str keywords: query to be searched on YouTube
        :param int limit: total playlists be searched
        :return: list of < playlist object > of each playlist regarding the query (consider limit)
        """
        raw = _src(f'https://www.youtube.com/results?search_query={_parser(keywords)}&sp=EgIQAw%253D%253D')
        found = re.findall(r"playlistId\":\"(.*?)\"", raw)
        pure = _filter(limit=limit, iterable=found)
        return _PlaylistBulk(pure) if pure else None



import re
from ._auxiliary import _src
from ._threads import _Thread


class _ChannelBulk:

    def __init__(self, iter: list):
        self._ls = iter

    @property
    def ids(self):
        return self._ls

    @property
    def urls(self):
        head = 'https://www.youtube.com/channel/'
        return [f'{head}{item}' for item in self._ls]
    
    @property
    def _sources(self):
        head = 'https://www.youtube.com/channel/'
        urls = [f'{head}{item}/about' for item in self._ls]

        def get_page(url):
            return _src(url)
        return _Thread.run(get_page, urls)
        
    @property
    def names(self):
        pattern = r"channelMetadataRenderer\":{\"title\":\"(.*?)\""
        return [re.findall(pattern, item)[0] for item in self._sources]

    @property
    def subscribers(self):
        pattern = r"}},\"simpleText\":\"(.*?) "
        temp = [re.findall(pattern, item) for item in self._sources]
        return [item[0] if item else None for item in temp]

    @property
    def views(self):
        pattern = r"viewCountText\":{\"simpleText\":\"(.*?)\""
        temp = [re.findall(pattern, item) for item in self._sources]
        return [item[0][:-6] if item else None for item in temp]

    @property
    def joined(self):
        pattern = r"text\":\"Joined \"},{\"text\":\"(.*?)\""
        temp = [re.findall(pattern, item) for item in self._sources]
        return [item[0] if item else None for item in temp]
        
    @property
    def countries(self):
        pattern = r"country\":{\"simpleText\":\"(.*?)\""
        temp = [re.findall(pattern, item) for item in self._sources]
        return [item[0] if item else None for item in temp]

    @property
    def custom_urls(self):
        pattern = r"canonicalChannelUrl\":\"(.*?)\""
        temp = [re.findall(pattern, item) for item in self._sources]
        return [item[0] if '/channel' not in item[0] else None for item in temp]

    @property
    def descriptions(self):
        pattern = r"description\":{\"simpleText\":\"(.*?)\""
        temp = [re.findall(pattern, item) for item in self._sources]
        return [item[0].replace('\\n', ' ') if item else None for item in temp]

    @property
    def avatars(self):
        pattern = "height\":88},{\"url\":\"(.*?)\""
        temp = [re.findall(pattern, item) for item in self._sources]
        return [item[0] if item else None for item in temp]

    @property
    def banners(self):
        pattern = r"width\":1280,\"height\":351},{\"url\":\"(.*?)\""
        temp = [re.findall(pattern, item) for item in self._sources]
        return [item[0] if item else None for item in temp]

    @property
    def verifieds(self):
        pattern = 'label":"Verified'
        return [True if re.search(pattern, item) else False for item in self._sources]

    @property
    def lives(self):
        pattern = r'{"text":" watching"}'
        return [True if re.search(pattern, item) else False for item in self._sources]



import re
from ._video import Video
from ._threads import _Thread
from ._videobulk import _VideoBulk
from ._auxiliary import _filter, _src
from ._playlistbulk import _PlaylistBulk


class Channel:

    def __init__(self, channelId: str):
        """
        :param str channelId: any of channel id, url , custom url
        """
        head = 'https://www.youtube.com/channel/'

        if '/channel/' in channelId:
            self._url = channelId
        elif '/c/' in channelId:
            self._url = channelId
        elif '/user/' in channelId:
            self._url = channelId
        else:
            self._url = head + channelId

    def __repr__(self):
        if self.name:
            return f'<Channel - {self.name}>'
        else:
            f'<Invalid ChannelObject>'

    @property
    def name(self):
        """
        :return: channel's name or None
        """
        raw = _src(f'{self._url}/about')
        name = re.findall("channelMetadataRenderer\":{\"title\":\"(.*?)\"", raw)
        return name[0] if name else None

    @property
    def valid(self):
        """
        :return: bool i.e. True if channel is valid else False
        """
        source = _src(self._url)
        return True if source else False

    @property
    def url(self):
        return self._url

    @property
    def id(self):
        """
        :return: the id of the channel
        """
        raw = _src(f'{self._url}/about')
        Id = re.search(r"\"channelId\":\"(.*?)\"", raw)
        if Id:
            return Id.group().replace('"', '').replace('channelId:', '')
        else:
            return

    @property
    def verified(self):
        """
        :return: bool i.e. True if channel is verified else False
        """
        raw = _src(f'{self._url}/about')
        isVerified = re.search(r'label":"Verified', raw)
        return True if isVerified else False

    @property
    def live(self):
        """
        :return: Bool of channel's Live Status
        """
        raw = _src(f'{self._url}/videos?view=2&live_view=501')
        check = re.findall("thumbnailOverlays\":\[(.*?)]", raw)
        return '{"text":"LIVE"}' in check[0]

    @property
    def livestream(self) -> str:
        """
        :return: channel's ongoing  livestream url
        """
        raw = _src(f'{self._url}/videos?view=2&live_view=501')
        check = re.findall("thumbnailOverlays\":\[(.*?)]", raw)
        if '{"text":"LIVE"}' in check[0]:
            Id = _filter(re.findall(r"videoId\":\"(.*?)\"", raw))[0]
            return f'https://www.youtube.com/watch?v={Id}'

    @property
    def livestreams(self) -> list:
        """
        :return: channel's ongoing  livestream urls
        """
        raw = _src(f'{self._url}/videos?view=2&live_view=501')
        if '{"text":" watching"}' in raw:
            Ids = _filter(re.findall(r"\"videoId\":\"(.*?)\"", raw))
            return [f'https://www.youtube.com/watch?v={Id}' for Id in Ids]

    @property
    def oldstreams(self) -> list:
        """
        :return: channel's old livestream urls
        """
        raw = _src(f'{self._url}/videos?view=2&live_view=503')
        Ids = _filter(re.findall(r"videoId\":\"(.*?)\"", raw))
        urls = [f'https://www.youtube.com/watch?v={Id}' for Id in Ids]
        return urls if urls else None

    def uploads(self, limit: int = None):
        """
        :param int limit: number of videos user wants from channel's latest upload
        :return: a < bulk video obj > of latest uploaded videos (consider limit)
        """
        raw = _src(f'{self._url}/videos?view=0&sort=dd&flow=grid')
        videos = _filter(re.findall(r"\"gridVideoRenderer\":{\"videoId\":\"(.*?)\"", raw), limit)
        return _VideoBulk(videos) if videos else None

    @property
    def latest(self):
        """
        :return: Channel's latest uploaded video in Video Object form
        """
        raw = _src(f'{self._url}/videos?view=0&sort=dd&flow=grid')
        thumbs = re.findall('thumbnails\":\[{\"url\":\"(.*?)\?', raw)
        ups = [url for url in thumbs if '_live' not in url]
        videoId = re.findall('i/(.*?)/', ups[0]) if ups else None
        return Video(videoId[0]) if videoId else None

    @property
    def subscribers(self):
        """
        :return: total number of subscribers the channel has or None
        """
        raw = _src(f'{self._url}/about')
        subs = re.findall("}},\"simpleText\":\"(.*?) ", raw)
        return subs[0] if subs else None

    @property
    def views(self):
        """
        :return: total number of views the channel got or None
        """
        raw = _src(f'{self._url}/about')
        viewList = re.findall(
            r"\"viewCountText\":{\"simpleText\":\"(.*?)\"}", raw
        )
        if viewList:
            return viewList[0].split(' ')[0]

    @property
    def joined(self):
        """
        :return: the channel creation date or None
        """
        raw = _src(f'{self._url}/about')
        joinList = re.findall(
            r"{\"text\":\"Joined \"},{\"text\":\"(.*?)\"}", raw
        )
        return joinList[0] if joinList else None

    @property
    def country(self):
        """
        :return: the name of the country from where the channel is or None
        """
        raw = _src(f'{self._url}/about')
        countryList = re.findall(
            r"\"country\":{\"simpleText\":\"(.*?)\"}", raw
        )
        return countryList[0] if countryList else None

    @property
    def custom_url(self):
        """
        :return: the custom _url of the channel or None
        """
        raw = _src(f'{self._url}/about')
        customList = re.findall(r"\"canonicalChannelUrl\":\"(.*?)\"", raw)
        customURL = customList[0] if customList else None
        if customURL:
            if '/channel/' not in customURL:
                return customURL

    @property
    def description(self):
        """
        :return: the existing description of the channel
        """
        raw = _src(f'{self._url}/about')
        descList = re.findall(r"{\"description\":{\"simpleText\":\"(.*?)\"}", raw)
        return descList[0].replace('\\n', '  ') if descList else None

    @property
    def avatar(self):
        """
        :return: logo / avatar url of the channel
        """
        raw = _src(f'{self._url}/about')
        data = re.findall("height\":88},{\"url\":\"(.*?)\"", raw)
        return data[0] if data else None

    @property
    def banner(self):
        """
        :return: banner url of the channel
        """
        raw = _src(f'{self._url}/about')
        data = re.findall(r"width\":1280,\"height\":351},{\"url\":\"(.*?)\"", raw)
        return data[0] if data else None

    @property
    def playlists(self):
        """
        :return: a list of < playlist object > for each public playlist the channel has
        """
        raw = _src(f'{self._url}/playlists')
        idList = re.findall(r"{\"url\":\"/playlist\?list=(.*?)\"", raw)
        return _PlaylistBulk(_filter(idList)) if idList else None

    @property
    def info(self):
        """
        :return: a dict containing channel info

        dict = {
            'name': -> str,
            'id': -> str,
            'subscribers': -> str,
            'verified': -> bool,
            'total_views': -> str,
            'joined_at': -> str,
            'country': -> str,
            'url': ->str,
            'custom_url': -> str,
            'avatar_url': -> str,
            'banner_url': -> str
            }

        """
        raw = _src(f'{self._url}/about')

        def extract(pattern):
            data = re.findall(pattern, raw)
            return data[0] if data else None

        patterns = [

            "channelMetadataRenderer\":{\"title\":\"(.*?)\"",
            "}},\"simpleText\":\"(.*?) ",
            "\"viewCountText\":{\"simpleText\":\"(.*?)\"}",
            "{\"text\":\"Joined \"},{\"text\":\"(.*?)\"}",
            "\"country\":{\"simpleText\":\"(.*?)\"}",
            "\"canonicalChannelUrl\":\"(.*?)\"",
            "height\":88},{\"url\":\"(.*?)\"",
            "width\":1280,\"height\":351},{\"url\":\"(.*?)\"",
            "channelId\":\"(.*?)\""

        ]

        ls = _Thread.run(extract, patterns)

        if ls[2]:
            views = ls[2].split(' ')[0]
        else:
            views = None

        curl = ls[5] if ls[5] and '/channel' not in ls[5] else None

        return {
            'name': ls[0],
            'id': ls[8],
            'subscribers': ls[1],
            'verified': self.verified,
            'views': views,
            'joined_at': ls[3],
            'country': ls[4],
            'url': self._url,
            'custom_url': curl,
            'avatar_url': ls[6],
            'banner_url': ls[7]
        }



from setuptools import setup


def readme():
    with open('README.md') as file:
        README = file.read()
    return README


setup(
    name='aiotube',
    version='1.4.2',
    description='Get YouTube Public Data without YouTubeAPI',
    long_description=readme(),
    long_description_content_type="text/markdown",
    package_dir={'aiotube': 'src'},
    packages=['aiotube'],
    install_requires=[
        'urllib3'
    ],
    url='https://github.com/jnsougata/AioTube/blob/main/README.md',
    project_urls={
        "Bug Tracker": "https://github.com/jnsougata/AioTube/issues"
    },
    author='Sougata Jana',
    author_email='jnsougata@gmail.com',
    license='MIT',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ],
    python_requires=">=3.6"
)



import urllib.request
from urllib.error import HTTPError
from collections import OrderedDict


def _src(url: str):
    """
    :param url: url to be requested
    :return: the requested page
    """
    try:
        with urllib.request.urlopen(url) as resp:
            return resp.read().decode()
    except HTTPError as status:
        if status.code == 404:
            raise RuntimeError('Invalid url')
        elif status.code == 429:
            raise RuntimeError('Too many requests')


def _filter(iterable: list, limit: int = None) -> list:
    """
    Restricts element repetition in iterable
    :param int limit: number of desired elements
    :param iterable: list or tuple of elements
    :return: modified list (consider limit)
    """
    if iterable:
        lim = limit if limit else len(iterable)
        converted = list(OrderedDict.fromkeys(iterable))
        if len(converted) - lim > 0:
            return converted[:-len(converted) + lim]
        else:
            return converted
    else:
        return []


def _duration(seconds: int):
    """
    :param seconds: duration to be converted
    :return: a duration string with 00h 00m 00s format
    """
    dur_hour = int(seconds // 3600)
    dur_min = int((seconds % 3600) // 60)
    dur_sec = int(seconds - (3600 * dur_hour) - (60 * dur_min))
    return f'{dur_hour}h {dur_min}m {dur_sec}s'


def _parser(kw: str):
    return kw.replace(" ", '+')
